import tkinter as tk
from tkinter import filedialog, messagebox
from PIL import Image, ImageTk

class ImageProjectorApp:
    def __init__(self, master):
        self.master = master
        self.master.title("Image Projector")

        # Variables
        self.left_image = None
        self.right_image = None
        self.alpha = 0.5  # Starting alpha value
        self.overlap = 200  # Overlapping region of 200 pixels
        self.left_offset_x = 0
        self.left_offset_y = 0
        self.right_offset_x = 0
        self.right_offset_y = 0
        self.side = "left"  # Can be 'left' or 'right'

        # Full-screen window for displaying images
        self.canvas = tk.Canvas(self.master, bg="black")
        self.canvas.pack(fill=tk.BOTH, expand=True)

        # Create control window
        self.create_control_window()

    def create_control_window(self):
        # Control window in a separate window
        self.control_window = tk.Toplevel(self.master)
        self.control_window.title("Controls")

        # GUI control 
        self.button_load_left = tk.Button(self.control_window, text="Load Left Image", command=self.load_left_image)
        self.button_load_left.pack()

        self.button_load_right = tk.Button(self.control_window, text="Load Right Image", command=self.load_right_image)
        self.button_load_right.pack()

        self.button_show_left = tk.Button(self.control_window, text="Show Left Image", command=self.show_left_image)
        self.button_show_left.pack()

        self.button_show_right = tk.Button(self.control_window, text="Show Right Image", command=self.show_right_image)
        self.button_show_right.pack()

        self.button_increase_alpha = tk.Button(self.control_window, text="Increase Alpha", command=self.increase_alpha)
        self.button_increase_alpha.pack()

        self.button_decrease_alpha = tk.Button(self.control_window, text="Decrease Alpha", command=self.decrease_alpha)
        self.button_decrease_alpha.pack()

        # Movement buttons for left image
        self.button_left_up = tk.Button(self.control_window, text="Left Up", command=self.move_left_up)
        self.button_left_up.pack()

        self.button_left_down = tk.Button(self.control_window, text="Left Down", command=self.move_left_down)
        self.button_left_down.pack()

        self.button_left_left = tk.Button(self.control_window, text="Left Left", command=self.move_left_left)
        self.button_left_left.pack()

        self.button_left_right = tk.Button(self.control_window, text="Left Right", command=self.move_left_right)
        self.button_left_right.pack()

        # Movement buttons for right image
        self.button_right_up = tk.Button(self.control_window, text="Right Up", command=self.move_right_up)
        self.button_right_up.pack()

        self.button_right_down = tk.Button(self.control_window, text="Right Down", command=self.move_right_down)
        self.button_right_down.pack()

        self.button_right_left = tk.Button(self.control_window, text="Right Left", command=self.move_right_left)
        self.button_right_left.pack()

        self.button_right_right = tk.Button(self.control_window, text="Right Right", command=self.move_right_right)
        self.button_right_right.pack()

    def load_left_image(self):
        file_path = filedialog.askopenfilename(title="Select Left Image", filetypes=[("PNG Files", "*.png")])
        if file_path:
            self.left_image = Image.open(file_path)
            self.show_image()

    def load_right_image(self):
        file_path = filedialog.askopenfilename(title="Select Right Image", filetypes=[("PNG Files", "*.png")])
        if file_path:
            self.right_image = Image.open(file_path)
            self.show_image()

    def show_image(self):
        if self.left_image and self.right_image:
            blended_image = self.blend_images()
            self.display_image(blended_image)

    def blend_images(self):
        if self.left_image.size[1] != self.right_image.size[1]:
            messagebox.showerror("Error", "Images must be of the same height")
            return None


        #Im not sure if this is the best option right now, ill adjust this later
        #
        width = self.left_image.size[0] + self.right_image.size[0] - self.overlap
        height = self.left_image.size[1]

        result = Image.new("RGB", (width, height))

        # Paste left image with offset
        left_position = (self.left_offset_x, self.left_offset_y)
        result.paste(self.left_image, left_position)

        # Crop and paste overlapping region with alpha blending
        left_overlap = self.left_image.crop((self.left_image.size[0] - self.overlap, 0, self.left_image.size[0], height))
        right_overlap = self.right_image.crop((0, 0, self.overlap, height))
        blended_overlap = Image.blend(left_overlap, right_overlap, self.alpha)
        overlap_position = (self.left_image.size[0] - self.overlap + self.left_offset_x, self.left_offset_y)
        result.paste(blended_overlap, overlap_position)

        # Paste right image with offset
        right_position = (self.left_image.size[0] + self.left_offset_x - self.overlap + self.right_offset_x, self.right_offset_y)
        result.paste(self.right_image, right_position)

        return result

    def display_image(self, image):
        if image:
            image_tk = ImageTk.PhotoImage(image)
            self.canvas.create_image(0, 0, anchor=tk.NW, image=image_tk)
            self.canvas.image = image_tk  # Keep a reference

    def show_left_image(self):
        if self.left_image:
            self.side = "left"
            self.display_image(self.left_image)

    def show_right_image(self):
        if self.right_image:
            self.side = "right"
            self.display_image(self.right_image)

    def increase_alpha(self):
        if self.alpha < 1.0:
            self.alpha += 0.05  # Increase by smaller steps for finer control
            self.alpha = round(self.alpha, 2)  # Ensure precision
        self.show_image()

    def decrease_alpha(self):
        if self.alpha > 0.0:
            self.alpha -= 0.05  # Decrease by smaller steps for finer control
            self.alpha = round(self.alpha, 2)  # Ensure precision
        self.show_image()

    def move_left_up(self):
        self.left_offset_y -= 10
        self.show_image()

    def move_left_down(self):
        self.left_offset_y += 10
        self.show_image()

    def move_left_left(self):
        self.left_offset_x -= 10
        self.show_image()

    def move_left_right(self):
        self.left_offset_x += 10
        self.show_image()

    def move_right_up(self):
        self.right_offset_y -= 10
        self.show_image()

    def move_right_down(self):
        self.right_offset_y += 10
        self.show_image()

    def move_right_left(self):
        self.right_offset_x -= 10
        self.show_image()

    def move_right_right(self):
        self.right_offset_x += 10
        self.show_image()

# Run the application
if __name__ == "__main__":
    root = tk.Tk()
    app = ImageProjectorApp(root)
    root.geometry("800x600")
    root.mainloop()
